<?php

/**
 * @link              https://elvez.co.jp
 * @since             1.0.0
 * @package           Elvez_Control_Access
 *
 * @wordpress-plugin
 * Plugin Name:       Elvez Control Access
 * Plugin URI:        https://wordpress.org/plugins/elvez-control-access
 * Description:       Control the access of non-logged-in users for each posts and pages.
 * Version:           1.1.4
 * Requires at least: 5.2
 * Requires PHP:      7.2
 * Author:            Elvez Inc,
 * Author URI:        https://elvez.co.jp
 * License:           GPL-2.0+
 * License URI:       http://www.gnu.org/licenses/gpl-2.0.txt
 * Text Domain:       elvez-control-access
 * Domain Path:       /languages
 */

// If this file is called directly, abort.
if ( ! defined( 'WPINC' ) ) {
	die;
}

/**
 * Currently plugin version.
 * Start at version 1.0.0 and use SemVer - https://semver.org
 * Rename this for your plugin and update it as you release new versions.
 */
define( 'ELVEZ_CONTROL_ACCESS_VERSION', '1.1.4' );

/**
 * The code that runs during plugin activation.
 * This action is documented in includes/class-elvez-control-access-activator.php
 */
function activate_elvez_control_access() {
	require_once plugin_dir_path( __FILE__ ) . 'includes/class-elvez-control-access-activator.php';
	Elvez_Control_Access_Activator::activate();
}

/**
 * The code that runs during plugin deactivation.
 * This action is documented in includes/class-elvez-control-access-deactivator.php
 */
function deactivate_elvez_control_access() {
	require_once plugin_dir_path( __FILE__ ) . 'includes/class-elvez-control-access-deactivator.php';
	Elvez_Control_Access_Deactivator::deactivate();
}

register_activation_hook( __FILE__, 'activate_elvez_control_access' );
register_deactivation_hook( __FILE__, 'deactivate_elvez_control_access' );

/**
 * The core plugin class that is used to define internationalization,
 * admin-specific hooks, and public-facing site hooks.
 */
require plugin_dir_path( __FILE__ ) . 'includes/class-elvez-control-access.php';

/**
 * Begins execution of the plugin.
 *
 * Since everything within the plugin is registered via hooks,
 * then kicking off the plugin from this point in the file does
 * not affect the page life cycle.
 *
 * @since    1.0.0
 */
function run_elvez_control_access() {

	$plugin = new Elvez_Control_Access();
	$plugin->run();

}
run_elvez_control_access();
